clear;
close all;
clc;

global gamma smin smax phiR kappa beta rog theta sshock ckrus

gamma  = 3;                  % elasticity between retailers

% CALIBRATED PARAMETERS

sa     = 0.0803;           % std. dev. of permanent productivity shocks
saa    = 0.8448;           % covariance of productivity shocks
alpha  = 0.1261;           % frequency of v shocks
rho    = 0.5244;           % probability stay in high z state
zbar   = 0.3007;           % max size of transitory productivity shocks
phiR   = 0.0199;           % menu costs for regular prices
kappa  = 0.0138;           % cost of deviating from regular price
pshock = 0.0296;

theta  = 1.01;
beta   = 0.96^(1/52)/exp((gamma-1)^2*sa^2/2);
rog    = 0.61^(1/4);     % consistent with earlier VAR evidence
sshock = sqrt(0.0018^2/((1+rog^6)+(1+rog^4)*(1+rog)^2+(1+rog^2)*(1+rog+rog^2)^2+(1+rog+rog^2+rog^3)^2));

% Discretize random variables

k = 11;  % points for persistent shocks

[e1,w1] = qnwnorm(k,0,1);
  e1    = sa*e1;  
  
w1 = w1*pshock;
w1((k-1)/2+1) = (1-pshock)+w1((k-1)/2+1);

e2 = e1;
w2 = w1;

ea=gridmake(e1,e2);
wa=gridmake(w1,w2);
wa=wa(:,1).*wa(:,2);
meane=mean(ea(:,1:2),2);

ea(:,1)=ea(:,1)+saa*meane;
ea(:,2)=ea(:,2)+saa*meane;

ea=ea(wa>1e-3,:); % do not compute low probability events
wa=wa(wa>1e-3,:);
wa=wa/sum(wa);

ez = [0;zbar];
e = gridmake(ea,ez);

w = gridmake(wa,[1-alpha;alpha]); w1=w(:,1).*w(:,2);
w = gridmake(wa,[1-rho; rho]);    w2=w(:,1).*w(:,2);

w = [w1,w2];

mumin = exp(-zbar+min(ea(:)))*gamma/(gamma-1)*0.85;
mumax = exp(+zbar+max(ea(:)))*gamma/(gamma-1)*1.15;

zmin = 0;
zmax = zbar;

%Bounds for state-space

gmin = -0.003;
gmax = +0.003;

smin=[mumin, mumin, zmin, gmin, -0.015];                       
smax=[mumax, mumax, zmax, gmax,  0.015];

% Approximation family:

n=[15,15,length(ez),3, 3];

fspace = fundef({'spli', nodeunif(n(1),smin(1),smax(1)),0,1},...
                {'spli', nodeunif(n(2),smin(2),smax(2)),0,1},...
                {'lin',  ez,0},...
                {'spli', nodeunif(n(4),smin(4),smax(4)),0,1},...
                {'spli', nodeunif(n(5),smin(5),smax(5)),0,1});           

grid = funnode(fspace);                       
s    = gridmake(grid);                             
ns   = length(s);

%Guess value function and decision rules:

mu = gamma/(gamma-1);
va = 2*(mu-1)*mu^(-gamma)/(1-beta*exp((gamma-1)*sa^2/2))*ones(ns,1);

c  = funfitxy(fspace,s,[va,va,va,va]);

% Few rounds of function iteration using golden search

load c;
load ckrus;

for it=1:3
 
cnew=c;

[v1,v2,v3] = saveBelmax(c,fspace,s,e,w);
c(:,1:3)=funfitxy(fspace,s,[v1,v2,v3]);
v4 = valfunc4(c,fspace,s,[],e,w);
c(:,4) = funfitxy(fspace,s,v4);

diff = [v1,v2,v3,v4]-repmat(v1,1,4);
diffnew= funeval(cnew,fspace,s)-repmat(funeval(cnew(:,1),fspace,s),1,4);

fprintf('%4i %6.2e %6.2e \n',[it,norm(c-cnew),norm(diff-diffnew)]);

if norm(c-cnew)<1e-5, break, end
end
    
% Few rounds of Newton using golden search

c = vec(c);

for it=1:15

cnew=c;  
[bel, beljac] =solvebel(cnew,fspace,s,e,w);
c=cnew-(beljac\bel);                               

fprintf('%4i %6.2e \n',[it,norm(c-cnew)]);  %report difference

if norm(c-cnew)<1e-7, break, end
end

clear beljac

%put coefficients back in 3 columns 
c=[c(1:ns,:),c(ns+1:2*ns,:),c(2*ns+1:3*ns,:),c(3*ns+1:4*ns,:)];
cnew = c;
save c c

fspacenew  = fspace;

[v1,v2,v3,v4,x1,x2] = saveBelmax(c,fspace,s,e,w);      % solve the firms' problem given correct coefficient

c_vx = funfitxy(fspacenew,s,[v1,v2,v3,x1,x2]);        % the reason I invoked saveBelmax here is to get optimal prices in state-space

v = funeval(c,fspace,s);


equilibrium_continuous


