clear;
close all;
clc;

global gamma smin smax phiR beta ckrus rog

gamma  = 3;               % elasticity between retailers

pshock = 1;

% CALIBRATED PARAMETERS

sa     = 0.0231;          % max size productivity shocks
phiR   = 0.0167;  


theta  = 1.01;
beta   = 0.96^(1/52)/exp((gamma-1)^2*sa^2/2);
rog    = 0.61^(1/4);     % consistent with earlier VAR evidence
sshock = sqrt(0.0018^2/((1+rog^6)+(1+rog^4)*(1+rog)^2+(1+rog^2)*(1+rog+rog^2)^2+(1+rog+rog^2+rog^3)^2));


k = 11;  % points for persistent shocks

[e1,w1] = qnwnorm(k,0,1);
  e1    = sa*e1;  
  
e = e1;
w = w1;

e=e(w>1e-3,:);
w=w(w>1e-3,:);

w=w/sum(w);

mumin = exp(min(e(:)))*gamma/(gamma-1)*.65;
mumax = exp(max(e(:)))*gamma/(gamma-1)*1.35;

gmin = - 0.003;
gmax = + 0.003;

smin=[mumin, gmin, -0.015];                       
smax=[mumax, gmax,  0.015];

% Approximation family:
 
n=[15, 3, 3];

fspace = fundef({'spli', nodeunif(n(1),smin(1),smax(1)),0,3},...
                {'spli', nodeunif(n(2),smin(2),smax(2)),0,1},...
                {'spli', nodeunif(n(3),smin(3),smax(3)),0,1});           

            
grid = funnode(fspace);                       
s    = gridmake(grid);                             
ns   = length(s);

% Guess value function and decision rules:

mu = gamma/(gamma-1);
va = (mu-1)*mu^(-gamma)/(1-beta*exp((gamma-1)*sa^2/2))*ones(ns,1);

c  = funfitxy(fspace,s,[va,va,va]);

load c;
load ckrus


for it=1:5
 
cnew=c;

[v1,v2] = saveBelmax(c,fspace,s,e,w);
c(:,1:2)=funfitxy(fspace,s,[v1,v2]);
v3 = valfunc3(c,fspace,s,[],e,w);
c(:,3) = funfitxy(fspace,s,v3);

fprintf('%4i %6.2e \n',[it,norm(c-cnew)]);

if norm(c-cnew)<1e-5, break, end
end
    
% Few rounds of Newton using golden search

c=vec(c);

for it=1:15

cnew=c;  
[bel, beljac] =solvebel(cnew,fspace,s,e,w);
c=cnew-(beljac\bel);                               

fprintf('%4i %6.2e \n',[it,norm(c-cnew)]);  %report difference

if norm(c-cnew)<1e-5, break, end
end

clear beljac

%put coefficients back in 3 columns 

c=[c(1:ns,:),c(ns+1:2*ns,:),c(2*ns+1:3*ns,:)];
cnew = c;
save c c

fspacenew  = fspace;

[v1,v2,v3,x] = saveBelmax(c,fspace,s,e,w);      % solve the firms' problem given correct coefficient

c_vx = funfitxy(fspacenew,s,[v1,v2,x]);        % the reason I invoked saveBelmax here is to get optimal prices in state-space

equilibrium_continuous


