clear;
clc;

% Simulate longer time-series of P,K, given decision rules we found with
% shorter time-series. Results will be less sensitive to variability of
% money shocks in shorter simulations


T = 100000;

freq = 1;    % 1 = weekly, 0 = monthly

% Table 3: Dominick's data

C = zeros(T,1);
g = zeros(T,1);

a1 = 0.946;
a2 = -0.668;

if freq

rog    = 0.61^(1/4);      % consistent with earlier VAR evidence
seg  = sqrt(0.0018^2/((1+rog^6)+(1+rog^4)*(1+rog)^2+(1+rog^2)*(1+rog+rog^2)^2+(1+rog+rog^2+rog^3)^2));

else
    
rog = 0.61;
seg = 0.0018;

end

for t = 2:T
    
   g(t) = rog*g(t-1)+randn*seg;
   C(t) = a1*C(t-1)+a2*g(t);
    
end

if freq

 Cm = reshape(C',4,T/4);   Cm = exp(Cm); Cm = sum(Cm)';   Cm = log(Cm/4);
 [Ct,Cd] = hptrend(Cm,14400);
 
else
 
[Ct,Cd] = hptrend(C,14400);

end

 disp('Aggregate Statistics')
 [corr([Cd(1:end-1),Cd(2:end)]),std(Cd)*100]