clear;
close all;
clc;

global gamma smin smax phiR beta mua ckrus rog

gamma  = 3;               % elasticity between retailers
mua    = 0;    
ha     = 0;
% CALIBRATED PARAMETERS

sa     = 0.112;           % max size productivity shocks
saa    = 0.4661;           % covariance of productivity shocks (weight on mean)
phiR   = 0.0107;          % menu costs for regular prices
pshock = 0.0786;

theta  = 1.01;
beta   = 0.96^(1/12)/exp((gamma-1)^2*sa^2/2);
rog    = 0.61;     % consistent with earlier VAR evidence
sshock = 0.0018;


% Discretize random variables

k = 11;  % points for persistent shocks

[e1,w1] = qnwnorm(k,0,1);
  e1    = sa*(1-2*ha)^(3/4)*e1.*exp(ha.*e1.^2/2);  

w1 = w1*pshock;
w1((k-1)/2+1) = (1-pshock)+w1((k-1)/2+1);

e2=e1;
w2=w1;

e=gridmake(e1,e2);
w=gridmake(w1,w2);
w=w(:,1).*w(:,2);
meane=mean(e(:,1:2),2);

e(:,1)=e(:,1)+saa*meane;
e(:,2)=e(:,2)+saa*meane;

e=e(w>1e-3,:);
w=w(w>1e-3,:);

w=w/sum(w);

mumin = exp(min(e(:)))*gamma/(gamma-1)*0.85;
mumax = exp(max(e(:)))*gamma/(gamma-1)*1.15;

%Bounds for state-space

gmin = -0.01;
gmax = +0.01;

smin=[mumin, mumin, gmin, -0.03];                       
smax=[mumax, mumax, gmax,  0.03];

% Approximation family:
 
n=[15, 15, 3, 3];

fspace = fundef({'spli', nodeunif(n(1),smin(1),smax(1)),0,3},...
                {'spli', nodeunif(n(2),smin(2),smax(2)),0,3},...
                {'spli', nodeunif(n(3),smin(3),smax(3)),0,1},...
                {'spli', nodeunif(n(4),smin(4),smax(4)),0,1});           

grid = funnode(fspace);                       
s    = gridmake(grid);                             
ns   = length(s);

% Guess value function and decision rules:

mu = gamma/(gamma-1);
va = 2*(mu-1)*mu^(-gamma)/(1-beta*exp((gamma-1)*sa^2/2))*ones(ns,1);

c  = funfitxy(fspace,s,[va,va,va]);

load c;
load ckrus


for it=1:5
 
cnew=c;

[v1,v2] = saveBelmax(c,fspace,s,e,w);
c(:,1:2)=funfitxy(fspace,s,[v1,v2]);
v3 = valfunc3(c,fspace,s,[],e,w);
c(:,3) = funfitxy(fspace,s,v3);

fprintf('%4i %6.2e \n',[it,norm(c-cnew)]);

if norm(c-cnew)<1e-5, break, end
end
    
% Few rounds of Newton using golden search

c=vec(c);

for it=1:15

cnew=c;  
[bel, beljac] =solvebel(cnew,fspace,s,e,w);
c=cnew-(beljac\bel);                               

fprintf('%4i %6.2e \n',[it,norm(c-cnew)]);  %report difference

if norm(c-cnew)<1e-5, break, end
end

clear beljac

%put coefficients back in 3 columns 

c=[c(1:ns,:),c(ns+1:2*ns,:),c(2*ns+1:3*ns,:)];
cnew = c;
save c c

fspacenew  = fspace;

[v1,v2,v3,x] = saveBelmax(c,fspace,s,e,w);      % solve the firms' problem given correct coefficient

c_vx = funfitxy(fspacenew,s,[v1,v2,x]);        % the reason I invoked saveBelmax here is to get optimal prices in state-space

equilibrium_continuous


