function  y=objective(x)


global  beta gamma mua smin smax phiR

% PARAMETER VALUES: 

sa     = x(1);             % max size productivity shocks
saa    = x(2);           % covariance of productivity shocks (weight on mean)
phiR   = x(3);           % menu costs for regular prices
alpha1 = x(4);
alpha2 = x(5);
prup   = x(6);

beta   = 0.96^(1/52)/exp((gamma-1)^2*sa^2/2);

x'

% Discretize random variables

k = 5;  % points for persistent shocks

[e1,w1]=qnwbeta(k,alpha1,alpha2);

e1=e1*sa;
e1=([-e1(end:-1:2);0;e1(2:end)]);
w1=[w1(end:-1:2)*(1-prup); w1(1); w1(2:end)*prup];

e2=e1;
w2=w1;

e=gridmake(e1,e2);
w=gridmake(w1,w2);
w=w(:,1).*w(:,2);
meane=mean(e(:,1:2),2);

e(:,1)=e(:,1)+saa*meane;
e(:,2)=e(:,2)+saa*meane;

e=e(w>1e-3,:);
w=w(w>1e-3,:);

w=w/sum(w);


mumin = exp(min(e(:)))*gamma/(gamma-1)*.85;
mumax = exp(max(e(:)))*gamma/(gamma-1)*1.15;

%Bounds for state-space

smin=[mumin, mumin];                       
smax=[mumax, mumax];

% Approximation family:

n=[11,11];

fspace = fundef({'spli', nodeunif(n(1),smin(1),smax(1)),0,3},...
                {'spli', nodeunif(n(2),smin(2),smax(2)),0,3});           

grid = funnode(fspace);                       
s    = gridmake(grid);                             
ns   = length(s);

%Guess value function and decision rules:

mu = gamma/(gamma-1);
va = 2*(mu-1)*mu^(-gamma)/(1-beta*exp((gamma-1)*sa^2/2))*ones(ns,1);

c  = funfitxy(fspace,s,[va,va,va]);

%load c;

for it=1:3
 
cnew=c;

[v1,v2] = saveBelmax(c,fspace,s,e,w);
c(:,1:2)=funfitxy(fspace,s,[v1,v2]);
v3 = valfunc3(c,fspace,s,[],e,w);
c(:,3) = funfitxy(fspace,s,v3);

fprintf('%4i %6.2e \n',[it,norm(c-cnew)]);

if norm(c-cnew)<1e-5, break, end
end
    
% Few rounds of Newton using golden search

c=vec(c);

for it=1:15

cnew=c;  
[bel, beljac] =solvebel(cnew,fspace,s,e,w);
c=cnew-(beljac\bel);                               

fprintf('%4i %6.2e \n',[it,norm(c-cnew)]);  %report difference

if norm(c-cnew)<1e-5, break, end
end

clear beljac

%put coefficients back in 3 columns 
c=[c(1:ns,:),c(ns+1:2*ns,:),c(2*ns+1:3*ns,:)];
cnew = c;
save c c

fspacenew  = fspace;

[v1,v2,v3,x] = saveBelmax(c,fspace,s,e,w);      % solve the firms' problem given correct coefficient

c_vx = funfitxy(fspacenew,s,[v1,v2,x]);        % the reason I invoked saveBelmax here is to get optimal prices in state-space

v = funeval(c,fspace,s);

simulate
filter_temporary
statistics_calibrate;


weight = ones(size(Ymodel));


y = weight'*(abs(Ymodel-Ydata)./Ydata*100)/sum(weight); 
