#delimit;

clear; 
set mem 1500m;
set matsize 2000;
set more off;
cd ..;
cd "C:\projects\multi-product firms\Econometrica Replication Files\REPLICATION FILES\MAIN CODE\Table_6_8_data\";
use dominicks;


rename col1 date;
rename col2 good;
rename col3 upc;
rename col4 cat;
rename col5 p;
rename col6 pR;
rename col7 q;
rename col8 c;


sort good date;
xtset good date;

g adj = p~=L.p;
replace adj = . if L.p==. | p==.;
g adjR = pR~=L.pR ;
replace adjR =. if L.pR==. | pR ==.;

g man=floor(upc/100000);  /*this is not really manufacturer, as 1st digit may be non-zero */
replace man=man-floor(man/100000)*100000 if man>100000;
drop if man==0; /*funny upc codes: generic cigarettes and products without brand etc. Notice this means fewer price changes (~ 10,000) than in Table 1 */


egen cost = filter(c), coef(1 1 1 1) lags(-3/0) normalise;
replace c = cost;

g change = ln(p/L.p);
g achange = abs(change);
g changeR = ln(pR/L.pR);
g achangeR = abs(changeR);
g markup = L.p/c;
g markupR = L.pR/c;

egen meanmarkup = mean(markup), by(good);
egen meanmarkupR = mean(markupR), by(good);

g lnmeanmarkupR = ln(meanmarkupR);
g lnmeanmarkup = ln(meanmarkup);

g dev =  abs(ln(markup/meanmarkup))^2;
g devR = abs(ln(markupR/meanmarkupR))^2;
g devS =  ln(markup/meanmarkup);
replace devS = abs(devS)^2*(devS>0);
drop if dev>1 | devR > 1 | devS>1;

egen count = count(p), by (good);


egen share = sum(q), by(good);
g lnshare = ln(share);
g psale = L.p~=L.pR;

g present  = share;

g adjshare = dev*share;
g adjRshare = devR*share;
g adjSshare = devS*share;


egen mancat = group(man cat); 

egen countmancat = count(present), by(mancat date);
egen countman=count(present), by(man date);
egen countcat=count(present), by(cat date);
egen countsto=count(present), by(date);

egen mancatadj=sum(adjshare), by(mancat date);
egen mancattot=sum(present),  by(mancat date);

egen manadj=sum(adjshare), by(man date);
egen mantot=sum(present),  by(man date);

egen catadj=sum(adjshare), by(cat date);
egen cattot=sum(present),  by(cat date);

egen storeadj=sum(adjshare), by(date);
egen storetot=sum(present),  by(date);

g fmancatadj = (mancatadj-adjshare)/(mancattot-share);
g fmanadj    = (manadj-mancatadj)/(mantot-mancattot);
g fcatadj=(catadj-mancatadj)/(cattot-mancattot);
g fstoreadj=(storeadj+mancatadj-manadj-catadj)/(storetot+mancattot-mantot-cattot);

*drop storeadj catadj manadj mancatadj;



egen mancatadjR=sum(adjRshare), by(mancat date);
egen manadjR=sum(adjRshare), by(man date);
egen catadjR=sum(adjRshare), by(cat date);
egen storeadjR=sum(adjRshare), by(date);

g fmancatadjR = (mancatadjR-adjRshare)/(mancattot-share);
g fmanadjR    = (manadjR-mancatadjR)/(mantot-mancattot);
g fcatadjR=(catadjR-mancatadjR)/(cattot-mancattot);
g fstoreadjR=(storeadjR+mancatadjR-manadjR-catadjR)/(storetot+mancattot-mantot-cattot);

drop storeadjR catadjR manadjR mancatadjR;

egen mancatsale=sum(adjSshare), by(mancat date);
egen mansale=sum(adjSshare), by(man date);
egen catsale=sum(adjSshare), by(cat date);
egen storesale=sum(adjSshare), by(date);

g fmancatsale = (mancatsale-adjSshare)/(mancattot-share);
g fmansale    = (mansale-mancatsale)/(mantot-mancattot);
g fcatsale=(catsale-mancatsale)/(cattot-mancattot);
g fstoresale=(storesale+mancatsale-mansale-catsale)/(storetot+mancattot-mantot-cattot);

drop storesale catsale mansale mancatsale;

g sale = p<pR & L.p==L.pR;

keep if count>=50 & countmancat>5 & countcat-countmancat>5 & countman-countmancat>5 & countsto - countman - countcat - countmancat>5;

dprobit adj lnshare dev psale lnmeanmarkup fmancatadj fmanadj fcatadj fstoreadj [pweight = share];
dprobit adjR lnshare devR psale lnmeanmarkupR fmancatadj fmanadj fcatadj fstoreadj  fmancatadjR fmanadjR fcatadjR fstoreadjR [pweight = share]; 

dprobit adjR lnshare devR psale lnmeanmarkupR [pweight = share];
dprobit adjR lnshare devR psale lnmeanmarkupR fmancatadj fmanadj fcatadj fstoreadj [pweight = share]; 
dprobit adjR lnshare devR psale lnmeanmarkupR fmancatadjR fmanadjR fcatadjR fstoreadjR [pweight = share]; 

dprobit sale lnshare devS lnmeanmarkupR [pweight = share] if L.p==L.pR;
dprobit sale lnshare devS lnmeanmarkupR fmancatsale fmanadj fcatadj fstoreadj [pweight = share] if L.p==L.pR; 
dprobit sale lnshare devS lnmeanmarkupR fmancatsale fmansale fcatsale fstoresale [pweight = share] if L.p==L.pR; 