clc;
disp('compute_equilibrium...');

T=550;

M = ones(T,1);
gM = zeros(T,1);

for t=2:T+500
     gM(t)   = rho*gM(t-1) + randn*sigma;
      M(t)   = exp(gM(t))*M(t-1);
end


%generate grid of states

Rgrid = nodeunif(101,smin(1),smax(1));
Pgrid = nodeunif(101,smin(2),smax(2));

edgesR = [Rgrid(1);[Rgrid(2:end)+Rgrid(1:end-1)]/2];
edgesP = [Pgrid(1);[Pgrid(2:end)+Pgrid(1:end-1)]/2];

K=501; 
demand=nodeunif(K,1e-7,1-1e-7);  %start with equidistributed sequence
demand=norminv(demand,0,sqrt(vbar));
demand=smax(2)*(demand>smax(2))+smin(2)*(demand<smin(2))+demand.*(demand>=smin(2)&demand<=smax(2));
edgesd=[demand(1:end-1)-sqrt(eps);demand(end)];

e=demand;
w=1/length(demand)*ones(size(demand));

state=gridmake(sgrid,e);

v=funeval(c_vx,fspace,state);
    
    v1=v(:,1);
    v2=v(:,2);
    x1=v(:,3:4);
    x2=v(:,5);
   
    psim=(x1(:,1).*(v1>=v2)+x2.*(v1<v2));
    isim=(x1(:,2).*(v1>=v2));
    stock=state(:,1);
    sales=min([exp(state(:,2)).*(psim./Pm).^(-gamma).*Cm,stock],[],2);
    ssim=(1-delta)*(stock-sales+isim);

wz=1/length(sgrid)*ones(length(sgrid),1);  %guess weights on ewach of the points in sgrid


P=zeros(T,1);
Fracorder=P;
Meanorder=P;
Imports=P;
Sales=P;
Inv=P;
IS=P;


for t=1:T
    
  wnew=wz;
  ww=gridmake(wnew,w); ww=ww(:,1).*ww(:,2);

 wjunk=hist2(ssim,state(:,2),edges,edgesd,ww);
 wz=sum(wjunk,2);
 wjunk=reshape(wjunk,size(wjunk,1)*size(wjunk,2),1);
 
 wguess=reshape(wjunk,size(wjunk,1)*size(wjunk,2),1);
 
 
fprintf('%4i %6.2e %6.2e %6.2e\n',[t,norm(wz-wnew),mean(abs(wnew(wnew>1e-2|wz>1e-2)-wz(wnew>1e-2|wz>1e-2))./wz(wnew>1e-2|wz>1e-2)),...
    mean(abs(wnew(wnew>1e-4|wz>1e-4)-wz(wnew>1e-4|wz>1e-4))./wz(wnew>1e-4|wz>1e-4))]);


P(t)=(wjunk'*psim.^(1-gamma)).^(1/(1-gamma));
Fracorder(t)=wjunk'*(isim>1e-5);
Imports(t)=wjunk'*isim;
Meanorder(t)=Imports(t)/Fracorder(t);
Sales(t)=wjunk'*sales;
Inv(t)=wjunk'*ssim;

end

   




p_all=cell(Lp,1);
a_all=cell(Lp,1);
z_all=cell(Lp,1);

parfor j=1:Lp

mu   =  zeros(N,T);
muR  =  zeros(N,T);
p    =  zeros(N,T);
pR   =  zeros(N,T);
cont =  zeros(N,T);
a    =  zeros(N,T);
z    =  zeros(N,T);
 
TRP  = [alpha0, 1-alpha0; alpha1, 1-alpha1];
cTRP = cumsum(TRP,2);
zmat = [0;vbar];                        % use to simulate process for z

randn('state',j+100);
rand('state',j+100);

shocka = norminv(nodeunif(N,.001,1-.001),0,sigmaa);
shocka = shocka/std(shocka)*sigmaa;

alive = nodeunif(N,0,1);   alive = alive < delta; 
unif = nodeunif(N,0,1);                              % use to simulate transition for v

for t = 1:T
    disp(t);
    if t==1
        
        shocka = shocka(randperm(N));
        alive = alive(randperm(N));
        unif  = unif(randperm(N));
        
        a(:,t) = astart;
        cont(:,t) = alive;
        
        zind   = (unif > pai) + 1;
        z(:,t) = zmat(zind);
        
        state = [gamma/(gamma-1)*ones(N,1),gamma/(gamma-1)*ones(N,1),gM(t)*ones(N,1),z(:,t)];
        
        v = funeval(c_vx,fspacenew,state);
        
        v(:,3) = v(:,3).*(state(:,1)>=smin(1) & state(:,1)<=smax(1));    % can't use old reg. price if out of bounds
        v(:,4) = v(:,4).*(state(:,2)>=smin(2) & state(:,2)<=smax(2));    % can't use old posted price if out of bounds
        v(:,5) = v(:,5).*(state(:,1)>=smin(1) & state(:,1)<=smax(1));    % can't use old reg. price if out of bounds
        
        v1   = v(:,1);
        v2   = v(:,2);
        v3   = v(:,3);
        v4   = v(:,4);
        v5   = v(:,5);
        
        xR   = v(:,6);
        xP   = v(:,7);
       
        TEMP = state(:,1)~=state(:,2);   % yesterday has a temp. price
        REG  = state(:,1)==state(:,2);   % yesterday had a REG
       
        % indicators for choice of REG guys
        
        R1=(v1>=max([v1 v2 v5],[],2));
        R2=(v2>=max([v1 v2 v5],[],2) & R1==0);
        R3=(v5>=max([v1 v2 v5],[],2) & R1==0 & R2==0);
        
        % indicators for choice of TEMP GUYS
        
        T1=(v1>=max([v1 v2 v3 v4],[],2));
        T2=(v2>=max([v1 v2 v3 v4],[],2) & T1==0);
        T3=(v3>=max([v1 v2 v3 v4],[],2) & T1==0 & T2==0);
        T4=(v4>=max([v1 v2 v3 v4],[],2) & T1==0 & T2==0 & T3==0);
        
        muR(:,t) = xR.*(REG & R1) + state(:,1).*(REG & ~ R1) + xR.*(TEMP & T1) + state(:,1).*(TEMP & ~T1); 
        mu(:,t)  = xR.*(REG & R1) + xP.*(REG & R2)+ state(:,1).*(REG & R3) + xR.*(TEMP & T1) + xP.*(TEMP & T2) + state(:,1).*(TEMP & T3) + state(:,2).*(TEMP & T4);
        
        pR(:,t) = muR(:,t).*Wss.*M(t)./exp(a(:,t))./exp(z(:,t));
        p(:,t)  = mu(:,t).*Wss.*M(t)./exp(a(:,t))./exp(z(:,t));
        
    else

        shocka = shocka(randperm(N));
        alive = alive(randperm(N));
        unif  = unif(randperm(N));
        
        a(:,t) = a(:,t-1) + shocka;
        cont(:,t) = alive;
        zind = sum(cTRP(zind,:) < repmat(unif,1,2),2)+1;
        
        z(:,t) = zmat(zind);
        
        state = [muR(:,t-1).*exp(shocka)/exp(gM(t)).*exp(z(:,t)-z(:,t-1)),mu(:,t-1).*exp(shocka)/exp(gM(t)).*exp(z(:,t)-z(:,t-1)),gM(t)*ones(N,1),z(:,t)];
        
        % Restart guys that do not survive
        
        state(~alive,:) = [gamma/(gamma-1)*ones(size(find(~alive))),gamma/(gamma-1)*ones(size(find(~alive))),gM(t)*ones(size(find(~alive))),z(~alive,t)];
        a(~alive,t)=zeros(size(find(~alive)));
           
        v = funeval(c_vx,fspacenew,state);
        
        v1   = v(:,1);
        v2   = v(:,2);
        v3   = v(:,3);
        v4   = v(:,4);
        v5   = v(:,5);
        
        xR   = v(:,6);
        xP   = v(:,7);
       
        TEMP = state(:,1)~=state(:,2);   % yesterday has a temp. price
        REG  = state(:,1)==state(:,2);   % yesterday had a REG
       
        % indicators for choice of REG guys
        
        R1=(v1>=max([v1 v2 v5],[],2));
        R2=(v2>=max([v1 v2 v5],[],2) & R1==0);
        R3=(v5>=max([v1 v2 v5],[],2) & R1==0 & R2==0);
        
        % indicators for choice of TEMP GUYS
        
        T1=(v1>=max([v1 v2 v3 v4],[],2));
        T2=(v2>=max([v1 v2 v3 v4],[],2) & T1==0);
        T3=(v3>=max([v1 v2 v3 v4],[],2) & T1==0 & T2==0);
        T4=(v4>=max([v1 v2 v3 v4],[],2) & T1==0 & T2==0 & T3==0);
        
        
        muR(:,t) = xR.*(REG & R1) + state(:,1).*(REG & ~ R1) + xR.*(TEMP & T1) + state(:,1).*(TEMP & ~T1); 
        mu(:,t)  = xR.*(REG & R1) + xP.*(REG & R2)+ state(:,1).*(REG & R3) + xR.*(TEMP & T1) + xP.*(TEMP & T2) + state(:,1).*(TEMP & T3) + state(:,2).*(TEMP & T4);
        
             
        pR(:,t) = muR(:,t).*Wss.*M(t)./exp(a(:,t))./exp(z(:,t));
        p(:,t)  = mu(:,t).*Wss.*M(t)./exp(a(:,t))./exp(z(:,t));
                
    end
end

p_all{j}=p;
a_all{j}=a;
z_all{j}=z;

end

clear R1 R2 R3 T1 T2 T3 T4 REG TEMP a alive astart cont mu muR p pR shocka unif v v1 v2 v3 v4 v5 v6 v7 xP xR z zind;


p=zeros(Lp*N,T);
a=p;
z=p;

for j=1:Lp
    
select=((j-1)*N+1:j*N)';

p(select,:)=p_all{j};
a(select,:)=a_all{j};
z(select,:)=z_all{j};

end


P = mean(p.^(1-gamma)).^(1/(1-gamma))';
C = M./P;


   